@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 import sys
 import os
+import inspect
 import yaml
 import pycurl
 import json
@@ -18,18 +19,22 @@ except ImportError:
 
 if is_root_folder:  # Inside the module
     from binding import Context
+    import generators
     from generators import parse_generator
     from parsing import flatten_dictionaries, lowercase_keys, safe_to_bool, safe_to_json
-    from validators import ValidationFailure
+    import validators
+    from validators import Failure
     from tests import Test, DEFAULT_TIMEOUT
-    from benchmarks import Benchmark, AGGREGATES, METRICS, build_benchmark
+    from benchmarks import Benchmark, AGGREGATES, METRICS, parse_benchmark
 else:  # Importing as library
     from pyresttest.binding import Context
+    import pyresttest.generators
     from pyresttest.generators import parse_generator
     from pyresttest.parsing import flatten_dictionaries, lowercase_keys, safe_to_bool, safe_to_json
-    from pyresttest.validators import ValidationFailure
+    import pyresttest.validators
+    from pyresttest.validators import Failure
     from pyresttest.tests import Test, DEFAULT_TIMEOUT
-    from pyresttest.benchmarks import Benchmark, AGGREGATES, METRICS, build_benchmark
+    from pyresttest.benchmarks import Benchmark, AGGREGATES, METRICS, parse_benchmark
 
 """
 Executable class, ties everything together into the framework.
@@ -91,7 +96,7 @@ class TestSet:
         self.benchmarks = list()
 
     def __str__(self):
-        return json.dumps(self, default=lambda o: safe_to_json)
+        return json.dumps(self, default=safe_to_json)
 
 class BenchmarkResult:
     """ Stores results from a benchmark for reporting use """
@@ -107,7 +112,7 @@ class BenchmarkResult:
         self.results = list()
 
     def __str__(self):
-        return json.dumps(self, default=lambda o: safe_to_json)
+        return json.dumps(self, default=safe_to_json)
 
 class TestResponse:
     """ Encapsulates everything about a test response """
@@ -122,7 +127,7 @@ class TestResponse:
         self.failures = list()
 
     def __str__(self):
-        return json.dumps(self, default=lambda o: safe_to_json(o))
+        return json.dumps(self, default=safe_to_json)
 
     def body_callback(self, buf):
         """ Write response body by pyCurl callback """
@@ -141,7 +146,7 @@ def read_test_file(path):
     teststruct = yaml.safe_load(os.path.expandvars(read_file(path)))
     return teststruct
 
-def build_testsets(base_url, test_structure, test_files = set(), working_directory = None):
+def parse_testsets(base_url, test_structure, test_files = set(), working_directory = None):
     """ Convert a Python datastructure read from validated YAML to a set of structured testsets
     The data stucture is assumed to be a list of dictionaries, each of which describes:
         - a tests (test structure)
@@ -175,7 +180,7 @@ def build_testsets(base_url, test_structure, test_files = set(), working_directo
                         test_files.add(importfile)
                         import_test_structure = read_test_file(importfile)
                         with cd(os.path.dirname(os.path.realpath(importfile))):
-                            import_testsets = build_testsets(base_url, import_test_structure, test_files)
+                            import_testsets = parse_testsets(base_url, import_test_structure, test_files)
                             testsets.extend(import_testsets)
                 elif key == u'url': #Simple test, just a GET to a URL
                     mytest = Test()
@@ -186,13 +191,13 @@ def build_testsets(base_url, test_structure, test_files = set(), working_directo
                 elif key == u'test': #Complex test with additional parameters
                     with cd(working_directory):
                         child = node[key]
-                        mytest = Test.build_test(base_url, child)
+                        mytest = Test.parse_test(base_url, child)
                         tests_out.append(mytest)
                 elif key == u'benchmark':
-                    benchmark = build_benchmark(base_url, node[key])
+                    benchmark = parse_benchmark(base_url, node[key])
                     benchmarks.append(benchmark)
                 elif key == u'config' or key == u'configuration':
-                    test_config = make_configuration(node[key])
+                    test_config = parse_configuration(node[key])
     testset = TestSet()
     testset.tests = tests_out
     testset.config = test_config
@@ -200,8 +205,8 @@ def build_testsets(base_url, test_structure, test_files = set(), working_directo
     testsets.append(testset)
     return testsets
 
-def make_configuration(node):
-    """ Convert input object to configuration information """
+def parse_configuration(node):
+    """ Parse input config to configuration information """
     test_config = TestConfig()
 
     node = lowercase_keys(flatten_dictionaries(node))  # Make it usable
@@ -266,7 +271,7 @@ def run_test(mytest, test_config = TestConfig(), context = None):
         curl.perform() #Run the actual call
     except Exception as e:
         # Curl exception occurred (network error), do not pass go, do not collect $200
-        result.failures.append(ValidationFailure(message="Curl Exception: {0}".format(e), details=str(e)))
+        result.failures.append(Failure(message="Curl Exception: {0}".format(e), details=str(e)))
         result.passed = False
         curl.close()
         return result
@@ -285,12 +290,12 @@ def run_test(mytest, test_config = TestConfig(), context = None):
         # Invalid response code
         result.passed = False
         failure_message = "Invalid HTTP response code: response code {0} not in expected codes [{1}]".format(response_code, mytest.expected_status)
-        result.failures.append(ValidationFailure(message=failure_message, details=None))
+        result.failures.append(Failure(message=failure_message, details=None))
 
     #print str(test_config.print_bodies) + ',' + str(not result.passed) + ' , ' + str(test_config.print_bodies or not result.passed)
 
     #Print response body if override is set to print all *OR* if test failed (to capture maybe a stack trace)
-    if test_config.print_bodies:
+    if test_config.print_bodies or not result.passed:
         if test_config.interactive:
             print "RESPONSE:"
         print result.body
@@ -302,10 +307,11 @@ def run_test(mytest, test_config = TestConfig(), context = None):
             logging.debug("executing this many validators: " + str(len(mytest.validators)))
             failures = result.failures
             for validator in mytest.validators:
-                validate_result = validator.validate(body, context=my_context)
+                # TODO add header parsing
+                validate_result = validator.validate(body=body, context=my_context)
                 if not validate_result:
                     result.passed = False
-                if isinstance(validate_result, ValidationFailure):
+                if isinstance(validate_result, Failure):
                     failures.append(validate_result)
                 # TODO add printing of validation for interactive mode
         else:
@@ -459,7 +465,7 @@ def metrics_to_tuples(raw_metrics):
 
 def write_benchmark_json(file_out, benchmark_result, benchmark, test_config = TestConfig()):
     """ Writes benchmark to file as json """
-    json.dump(benchmark_result, file_out, default=lambda o: safe_to_json)
+    json.dump(benchmark_result, file_out, default=safe_to_json)
 
 def write_benchmark_csv(file_out, benchmark_result, benchmark, test_config = TestConfig()):
     """ Writes benchmark to file as csv """
@@ -479,7 +485,7 @@ def write_benchmark_csv(file_out, benchmark_result, benchmark, test_config = Tes
 # Method to call when writing benchmark file
 OUTPUT_METHODS = {u'csv' : write_benchmark_csv, u'json': write_benchmark_json}
 
-def execute_testsets(testsets):
+def run_testsets(testsets):
     """ Execute a set of tests, using given TestSet list input """
     group_results = dict() #results, by group
     group_failure_counts = dict()
@@ -577,6 +583,26 @@ def execute_testsets(testsets):
 
     return total_failures
 
+def register_extensions(modules):
+    """ Import the modules and register their respective extensions """
+    for ext in modules:
+        module = __import__(ext)
+
+        # Extensions are registered by applying a register function to sets of registry name/function pairs inside an object
+        extension_applies = {
+            'VALIDATORS': validators.register_validator,
+            'COMPARATORS': validators.register_comparator,
+            'VALIDATOR_TESTS': validators.register_test,
+            'EXTRACTORS': validators.register_extractor,
+            'GENERATORS': generators.register_generator
+        }
+
+        for registry_name, register_function in extension_applies.items():
+            if hasattr(module, registry_name):
+                registry = getattr(module, registry_name)
+                for key, val in registry.items():
+                    register_function(key, val)
+
 def main(args):
     """
     Execute a test against the given base url.
@@ -592,20 +618,29 @@ def main(args):
     if 'log' in args and args['log'] is not None:
         logging.basicConfig(level=LOGGING_LEVELS.get(args['log'].lower(), logging.NOTSET))
 
+    if 'import_extensions' in args and args['import_extensions']:
+        extensions = args['import_extensions'].split(';')
+
+        # We need to add current folder to working path to import modules
+        working_folder = args['cwd']
+        if working_folder not in sys.path:
+            sys.path.insert(0, working_folder)
+        register_extensions(extensions)
+
     test_file = args['test']
     test_structure = read_test_file(test_file)
-    tests = build_testsets(args['url'], test_structure, working_directory=os.path.dirname(test_file))
+    tests = parse_testsets(args['url'], test_structure, working_directory=os.path.dirname(test_file))
 
     # Override configs from command line if config set
     for t in tests:
-        if 'print_bodies' in args and args['print_bodies'] is not None and not bool(args['print_bodies']):
+        if 'print_bodies' in args and args['print_bodies'] is not None and bool(args['print_bodies']):
             t.config.print_bodies = safe_to_bool(args['print_bodies'])
 
         if 'interactive' in args and args['interactive'] is not None:
             t.config.interactive = safe_to_bool(args['interactive'])
 
     # Execute all testsets
-    failures = execute_testsets(tests)
+    failures = run_testsets(tests)
 
     sys.exit(failures)
 
@@ -617,6 +652,7 @@ if(__name__ == '__main__'):
     parser.add_option(u"--interactive", help="Interactive mode", action="store", type="string")
     parser.add_option(u"--url", help="Base URL to run tests against", action="store", type="string")
     parser.add_option(u"--test", help="Test file to use", action="store", type="string")
+    parser.add_option(u'--import_extensions', help='Extensions to import, separated by semicolons', action="store", type="string")
 
     (args, unparsed_args) = parser.parse_args()
     args = vars(args)
@@ -634,4 +670,5 @@ if(__name__ == '__main__'):
             parser.print_help()
             parser.error("wrong number of arguments, need both url and test filename, either as 1st and 2nd parameters or via --url and --test")
 
+    args['cwd'] = os.path.realpath(os.path.abspath(os.getcwd()))  # So modules can be loaded from current folder
     main(args)
\ No newline at end of file
